from evoprompt.cli import argument_parser
from evoprompt.evolution.evolution import (
    DifferentialEvolution,
    DifferentialEvolutionWithCot,
    EvolutionAlgorithm,
    GeneticAlgorithm,
)
from evoprompt.utils import get_all_subclasses

optimizers = {
    algorithm.shorthand: algorithm
    for algorithm in get_all_subclasses(EvolutionAlgorithm)
}


def get_optimizer_class(name: str) -> type[EvolutionAlgorithm]:
    if name not in optimizers:
        raise ValueError("Optimization Algorithm %s does not exist", name)
    return optimizers[name]


argument_parser.add_argument(
    "--evolution-algorithm", "-a", type=str, choices=optimizers.keys(), default="ga"
)
argument_parser.add_argument(
    "--use-evolution-demo",
    action="store_true",
    help="Whether to prepend a single demonstration example for evolution or not",
)
