from evoprompt.cli import argument_parser
from evoprompt.models import LLMModel

# make sure to run definitions of subclasses of Task first
from evoprompt.task.task import EvaluationStrategyKey, Task
from evoprompt.task.question_answering import QuestionAnswering
from evoprompt.task.text_classification import TextClassification
from evoprompt.task.sentiment_analysis import SentimentAnalysis
from evoprompt.task.topic_classification import AGNews, TREC
from evoprompt.task.subjectivity_classification import Subj
from evoprompt.task.text_generation import TextGeneration
from evoprompt.task.summarization import Summarization, SAMSum
from evoprompt.task.simplification import Simplification, ASSET

from evoprompt.utils import get_all_subclasses

# at this point, we assume that all subclasses of Task have been defined
tasks = {
    task.shorthand: task
    for task in get_all_subclasses(Task)
    if hasattr(task, "shorthand")
}


def get_task(name: str, evaluation_model: LLMModel, **options) -> Task:
    if name not in tasks:
        raise ValueError(
            f"Task %{name} does not exist; available tasks: {list(tasks.keys())}"
        )
    return tasks[name](evaluation_model, **options)


argument_parser.add_argument("--debug", "-d", action="store_true", default=None)
argument_group = argument_parser.add_argument_group("Task arguments")
argument_group.add_argument(
    "--task", type=str, required=True, choices=sorted(tasks.keys())
)
argument_group.add_argument("--use-grammar", "-g", action="store_true")
argument_group.add_argument(
    "--evaluation-strategy",
    choices=EvaluationStrategyKey.__args__,
    default="simple",
)
argument_group.add_argument(
    "--n-evaluation-demo",
    type=int,
    help="Number of demonstration examples per class used for evaluation",
)
