import logging

from evaluate import load as load_metric

from evoprompt.task.base_prompts import BasePromptsFromJsonMixin
from evoprompt.task import TextGeneration
from evoprompt.task.task import DatasetDatum

logger = logging.getLogger(__name__)


class Simplification(TextGeneration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

        self.metric = load_metric("evaluate-metric/sari")

    def compute_metric(self, datum: DatasetDatum, prediction: str) -> float:
        gold_label = self._get_gold_label_generation_for_datum(datum)
        return self.metric.compute(
            sources=[self._get_text_for_datum(datum)],
            predictions=[prediction],
            references=[gold_label],
        )["sari"]

    @staticmethod
    def _get_prediction_prefix():
        return "Simplification: "

    @property
    def metric_name(self):
        return "sari"


class ASSET(BasePromptsFromJsonMixin, Simplification):
    shorthand = "asset"
    base_prompts_files = [
        "evoprompt/initial_prompts/asset/prompts.json",
        "evoprompt/initial_prompts/asset/prompts_auto.json",
    ]

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(
            *args,
            validation_dataset="facebook/asset",
            test_dataset="facebook/asset",
            validation_split="validation",
            test_split="test",
            **kwargs
        )

    @staticmethod
    def _get_text_for_datum(datum: DatasetDatum) -> str:
        return datum["original"]

    @staticmethod
    def _get_gold_label_generation_for_datum(datum: DatasetDatum) -> str:
        return datum["simplifications"]
