import logging

from evaluate import load as load_metric

from evoprompt.task.base_prompts import BasePromptsFromJsonMixin
from evoprompt.task import TextGeneration
from evoprompt.task.task import DatasetDatum

logger = logging.getLogger(__name__)


class Summarization(TextGeneration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

        self.metric = load_metric("evaluate-metric/rouge")

    def compute_metric(self, datum: DatasetDatum, prediction: str) -> float:
        gold_label = self._get_gold_label_generation_for_datum(datum)
        return (
            self.metric.compute(predictions=[prediction], references=[gold_label])[
                "rougeL"
            ]
            * 100
        )

    @staticmethod
    def _get_prediction_prefix():
        return "Summary: "

    @property
    def metric_name(self):
        return "rougeL"


class SAMSum(BasePromptsFromJsonMixin, Summarization):
    shorthand = "sams"
    base_prompts_files = [
        "evoprompt/initial_prompts/sam/prompts.json",
        "evoprompt/initial_prompts/sam/prompts_auto.json",
    ]

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(
            *args,
            validation_dataset="Samsung/samsum",
            test_dataset="Samsung/samsum",
            validation_split="train",
            test_split="test",
            **kwargs
        )

    @staticmethod
    def _get_text_for_datum(datum: DatasetDatum) -> str:
        return datum["dialogue"]

    @staticmethod
    def _get_gold_label_generation_for_datum(datum: DatasetDatum) -> str:
        return datum["summary"]
